"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var XmlConverter =
/*#__PURE__*/
function () {
  function XmlConverter() {
    _classCallCheck(this, XmlConverter);
  }

  _createClass(XmlConverter, [{
    key: "convert",

    /**
     * Converts an object to an xml string
     *
     * @param {object} object The source object
     *
     * @return {string} XML string result
     */
    value: function convert(object) {
      this.indentLevel = -1;
      var xmlStringParts = ['<?xml version="1.0" encoding="UTF-8"?>', this.convertObject(object)];
      return xmlStringParts.join('');
    }
    /**
     * Processes an object
     *
     * @private
     *
     * @param {object} object The source object
     *
     * @return {string} XML string result
     */

  }, {
    key: "convertObject",
    value: function convertObject(object) {
      var _this = this;

      var xml = [];

      _lodash["default"].forOwn(object, function (value, key) {
        if (_lodash["default"].isNumber(value) || !_lodash["default"].isEmpty(value)) {
          _this.indentLevel++;
          xml.push(_this.processObjectProperty(key, value));
          _this.indentLevel--;
        }
      });

      return xml.join('');
    }
    /**
     * Processes an object property with its key
     *
     * @private
     *
     * @param {string} key Object key
     * @param {string|array|object} value Object value
     *
     * @return {string} XML string result
     */

  }, {
    key: "processObjectProperty",
    value: function processObjectProperty(key, value) {
      if (_lodash["default"].isPlainObject(value)) {
        var convertedObject = this.convertObject(value);
        return !_lodash["default"].isEmpty(convertedObject) ? this.createNode(key, convertedObject) : '';
      }

      if (_lodash["default"].isArray(value)) {
        return this.convertArray(key, value);
      }

      return this.createNode(key, this.escapeValue(value));
    }
    /**
     * Processes an array
     *
     * @private
     *
     * @param {string} key The xml node key
     * @param {array} arr The source array
     *
     * @return {string} XML string result
     */

  }, {
    key: "convertArray",
    value: function convertArray(key, arr) {
      var _this2 = this;

      var xml = [];

      _lodash["default"].each(arr, function (item) {
        if (_lodash["default"].isPlainObject(item)) {
          return xml.push(_this2.createNode(key, _this2.convertObject(item)));
        }

        return xml.push(_this2.createNode(key, _this2.escapeValue(item)));
      });

      return xml.join('');
    }
    /**
     * Creates a single xml node
     *
     * @private
     *
     * @param {string} key Node identifier
     * @param {string} value Node content
     *
     * @return {string} XML string result
     */

  }, {
    key: "createNode",
    value: function createNode(key, value) {
      var indentation = _lodash["default"].pad('', this.indentLevel * this.indentationSpaces);

      if (this.indentationSpaces > 0) {
        indentation = "\n".concat(indentation);
      }

      if (value.toString().indexOf('</') !== -1) {
        value += indentation;
      }

      return "".concat(indentation, "<").concat(key, ">").concat(value, "</").concat(key, ">");
    }
    /**
     * Escapes a string value for propper xml formatting
     *
     * @private
     *
     * @param {string} str The target string value
     *
     * @return {string} Escaped string
     */

  }, {
    key: "escapeValue",
    value: function escapeValue(str) {
      return !_lodash["default"].isString(str) ? str : str.replace(/[<>&'"]/g, function (_char) {
        /* eslint-disable */
        switch (_char) {
          case '<':
            return '&lt;';

          case '>':
            return '&gt;';

          case '&':
            return '&amp;';

          case '\'':
            return '&apos;';

          case '"':
            return '&apos;';
        }
        /* eslint-enable */

      });
    }
  }], [{
    key: "toXml",

    /**
     * Converts an object to an xml string | static call
     *
     * @param {object} object The source object
     * @param {integer} indentationSpaces Number of spaces used for indentation
     *
     * @return {string} XML string result
     */
    value: function toXml(object) {
      var indentationSpaces = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var newInstance = new XmlConverter();
      newInstance.indentationSpaces = indentationSpaces;
      return newInstance.convert(object);
    }
  }]);

  return XmlConverter;
}();

exports["default"] = XmlConverter;

